@@ -671,7 +671,7 @@ hex2ba_core(bitarrayobject *a, Py_buffer hexstr)
                          "base16, got '%c' (0x%02x)", c, c);
             return -1;
         }
-        assert(0 <= x && x < 16);
+        assert(x >> 4 == 0);
         a->ob_item[i / 2] |= x << 4 * ((i + be) % 2);
         i++;
     }
@@ -723,29 +723,33 @@ static const char base32_alphabet[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ234567";
 static const char base64_alphabet[] =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
+/* Given the length of the base m in [1..6] and a character c, return
+   its index in the base 2**m alphabet , or -1 if when c is not included.
+   Note: i >> m checks if i is in range(0, 2**m) */
 static int
-digit_to_int(int n, char c)
+digit_to_int(int m, char c)
 {
-    static signed char table[2][256];
+    static signed char table[2][128];
     static int setup = 0;
     int i;
 
-    if (n <= 16) {                               /* base 2, 4, 8, 16 */
+    if (m < 5) {                                 /* base 2, 4, 8, 16 */
         i = hex_to_int(c);
-        if (i < n)
-            return i;
-        return -1;
+        return i >> m ? -1 : i;
     }
+
+    if (0x80 & c)  /* non-ASCII */
+        return -1;
+
     if (!setup) {
-        for (i = 0; i < 256; i++)
-            table[0][i] = table[1][i] = -1;
+        memset(table, 0xff, sizeof table);  /* (signed char) 0xff -> -1 */
         for (i = 0; i < 32; i++)
             table[0][(unsigned char) base32_alphabet[i]] = i;
         for (i = 0; i < 64; i++)
             table[1][(unsigned char) base64_alphabet[i]] = i;
         setup = 1;
     }
-    return table[n / 64][(unsigned char) c];     /* base 32, 64 */
+    return table[m - 5][(unsigned char) c];      /* base 32, 64 */
 }
 
 /* return m = log2(n) for m in [1..6] */
@@ -799,6 +803,7 @@ ba2base_core(bitarrayobject *a, int m, Py_ssize_t group, char *sep)
             int q = le ? k : (m - k - 1);
             x |= getbit(a, i * m + k) << q;
         }
+        assert(x >> m == 0);
         str[j++] = alphabet[x];
     }
     assert(j == strsize);
@@ -865,23 +870,23 @@ static int
 base2ba_core(bitarrayobject *a, Py_buffer asciistr, int m)
 {
     const char *str = asciistr.buf;
-    const int le = IS_LE(a), n = 1 << m;
+    const int le = IS_LE(a);
     Py_ssize_t i = 0, j;
 
     assert(a->nbits == asciistr.len * m && 1 <= m && m <= 6);
 
     for (j = 0; j < asciistr.len; j++) {
         unsigned char c = str[j];
-        int k, x = digit_to_int(n, c);
+        int k, x = digit_to_int(m, c);
 
         if (x < 0) {
             if (is_whitespace(c))
                 continue;
             PyErr_Format(PyExc_ValueError, "invalid digit found for "
-                         "base%d, got '%c' (0x%02x)", n, c, c);
+                         "base%d, got '%c' (0x%02x)", 1 << m, c, c);
             return -1;
         }
-        assert(0 <= x && x < n);
+        assert(x >> m == 0);
         for (k = 0; k < m; k++) {
             int q = le ? k : (m - k - 1);
             setbit(a, i++, x & (1 << q));
@@ -898,7 +903,7 @@ base2ba(PyObject *module, PyObject *args, PyObject *kwds)
     PyObject *endian = Py_None;
     Py_buffer asciistr;
     bitarrayobject *a = NULL;
-    int m, n, t;                   /* n = 2^m */
+    int m, n, t;                   /* n = 2**m */
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "is*|O:base2ba", kwlist,
                                      &n, &asciistr, &endian))
@@ -940,7 +945,7 @@ static int
 next_char(PyObject *iter)
 {
     PyObject *item;
-    unsigned char c;
+    Py_ssize_t v;
 
     item = PyIter_Next(iter);
     if (item == NULL) {
@@ -950,16 +955,17 @@ next_char(PyObject *iter)
         return -1;
     }
 
-    if (!PyLong_Check(item)) {
-        PyErr_Format(PyExc_TypeError, "int iterator expected, "
-                     "got '%s' element", Py_TYPE(item)->tp_name);
-        Py_DECREF(item);
+    v = PyNumber_AsSsize_t(item, NULL);
+    Py_DECREF(item);
+    if (v == -1 && PyErr_Occurred())
+        return -1;
+
+    if (v >> 8) {
+        PyErr_Format(PyExc_ValueError,
+                     "byte must be in range(0, 256), got: %zd", v);
         return -1;
     }
-
-    c = (unsigned char) PyLong_AsLong(item);
-    Py_DECREF(item);
-    return (int) c;
+    return (int) v;
 }
 
 /* write n bytes (into buffer str) representing non-negative integer i,
@@ -1085,8 +1091,8 @@ sc_calc_rts(bitarrayobject *a)
     Py_ssize_t *res, m;
 
     memset(zeros, 0x00, SEGSIZE);
-    res = (Py_ssize_t *) PyMem_Malloc((size_t)
-                                      sizeof(Py_ssize_t) * (n_seg + 1));
+    res = (Py_ssize_t *) PyMem_Malloc((size_t) sizeof(Py_ssize_t) *
+                                      (n_seg + 1));
     if (res == NULL)
         return (Py_ssize_t *) PyErr_NoMemory();
 
@@ -1202,7 +1208,7 @@ sc_write_raw(char *str, bitarrayobject *a, Py_ssize_t *rts, Py_ssize_t offset)
     assert((k >= 32 || k == nbytes) && (k <= 32 || k % 32 == 0));
 
     /* block header */
-    *str = (char) (k <= 32 ? k : (k / 32) + 31);
+    *str = (char) (k <= 32 ? k : k / 32 + 31);
 
     /* block data */
     assert(offset + k <= Py_SIZE(a));
